"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addLinks = exports.mergeTrees = exports.buildTree = exports.getLocalId = void 0;
const types_1 = require("../types/types");
const addToPath = (id, tree, modulePath, node) => {
    if (modulePath.length === 0) {
        throw new Error(`Error adding node to path ${id}`);
    }
    const [head, ...rest] = modulePath;
    if (rest.length === 0) {
        tree.children.push({ ...node, name: head });
        return;
    }
    else {
        let newTree = tree.children.find((folder) => folder.name === head && types_1.isModuleTree(folder));
        if (!newTree) {
            newTree = { name: head, children: [] };
            tree.children.push(newTree);
        }
        addToPath(id, newTree, rest, node);
        return;
    }
};
const getLocalId = (projectRoot, moduleId) => {
    return moduleId.replace(projectRoot, "");
};
exports.getLocalId = getLocalId;
const buildTree = (projectRoot, modules, mapper) => {
    const tree = {
        name: "root",
        children: [],
    };
    for (const { id, renderedLength, gzipLength, brotliLength } of modules) {
        const localId = exports.getLocalId(projectRoot, id);
        const mod = { id: localId, renderedLength, gzipLength, brotliLength };
        const uid = mapper.setValueByModuleId(localId, mod);
        const nodeData = { uid };
        const pathParts = localId.split(/\\|\//).filter((p) => p !== "");
        addToPath(localId, tree, pathParts, nodeData);
    }
    return tree;
};
exports.buildTree = buildTree;
const mergeTrees = (trees) => {
    const newTree = {
        name: "root",
        children: trees,
        isRoot: true,
    };
    return newTree;
};
exports.mergeTrees = mergeTrees;
const addLinks = (projectRoot, startModuleId, getModuleInfo, links, mapper) => {
    const processedNodes = {};
    const moduleIds = [startModuleId];
    while (moduleIds.length > 0) {
        const moduleId = moduleIds.shift();
        const localId = exports.getLocalId(projectRoot, moduleId);
        const moduleUid = mapper.getUid(localId);
        if (processedNodes[moduleUid]) {
            continue;
        }
        else {
            processedNodes[moduleUid] = true;
        }
        const mod = mapper.getValue(moduleUid, { id: localId, renderedLength: 0 });
        const moduleInfo = getModuleInfo(moduleId);
        if (!moduleInfo) {
            return;
        }
        if (moduleInfo.isEntry) {
            mod.isEntry = true;
        }
        if (moduleInfo.isExternal) {
            mod.isExternal = true;
        }
        for (const importedId of moduleInfo.importedIds) {
            const localId = exports.getLocalId(projectRoot, importedId);
            const importedUid = mapper.getUid(localId);
            links.push({ source: moduleUid, target: importedUid });
            moduleIds.push(importedId);
        }
        for (const importedId of moduleInfo.dynamicallyImportedIds) {
            const localId = exports.getLocalId(projectRoot, importedId);
            const importedUid = mapper.getUid(localId);
            links.push({ source: moduleUid, target: importedUid, dynamic: true });
            moduleIds.push(importedId);
        }
    }
};
exports.addLinks = addLinks;
